%% JANIS - Just another neuroimaging software
% author: Dominik Grotegerd
% Neuroimaging Group @University Hospital Münster
% License: BSD
%
% Commands:
%   janis                  - starts JANIS Graphical User Interface
%   janis('version')       - returns actual version number
%   janis('license')       - returns licence
%
% Version Changes          -> changelog.txt
% Documentation            -> Directory: doc
% third party software     -> Directory: extlib
function varargout = janis(varargin)

% Constants
versionNr = '2.3';
versionYear = '2013';
author = 'Dominik Grotegerd';
institute = 'Neuroimaging Group, Dep. of Psychiatry, University Hospital Muenster, Germany';
groupURL = 'http://www.uni-muenster.de/neuroimage';


[janis_path ign] = fileparts(which('janis'));
% setup path & launch GUI
if isempty(varargin)
    addpath(janis_path);
    addpath([janis_path filesep 'oo']);
    addpath([janis_path filesep 'gui']);
    addpath([janis_path filesep 'util']);
    addpath(genpath([janis_path filesep 'extlib']));
    janis_gui_main
% return infos
elseif isstr(varargin{1})
    switch varargin{1}
        case 'version'
            fprintf([janis('getVersionString') '\n']);
        case 'getVersionString'
            varargout{1} = ['JANIS version ' versionNr ' (' versionYear ')'];
        case 'license'
            lic = janis('getLicenseString');
            cellfun(@(x) fprintf([x '\n']),lic{:});
        case 'getLicenseString'
            fid = fopen([janis_path filesep 'license.txt']);
            varargout{1} = textscan(fid, '%s','delimiter', '\n');
            fclose(fid);
        case 'getAuthor'
            varargout{1} = author;
        case 'group'
            varargout{1} = institute;
        case 'www'
            varargout{1} = groupURL;
        otherwise
            error('Unkown option');
    end
else
    error('Unkown parameter');
end




