classdef janis_classifier < handle
    % janis_classifier
    %   The interface for every classification algorithm
    
    properties (SetAccess = protected)
       classifierStruct;  % Some kind of a trained classifier (how this is realized in different classifiers, may vary!
    end
    
    properties (SetAccess = public)
       title = 'unnamed';           % a description for classifier 
    end
    
    methods (Abstract)
        % method to train a classifier
        this = train(this,trainPatterns,trainLabels)       
        % method to evaluate classifier with testPatterns
        % estimations is a vector of label predictions
        % decFactors is a struct holding more estimates and parameters
        % influencing the predictions (like probabilities, decision values)
        % and varies depeding on the classifier 
        [estimations decFactors] = test(this,testPatterns);
        % returns a trained classifier, e.g. for map extraction
        clStruct = getClassifier(this,trainPatterns,trainLabels); 
        % A method with a simple GUI dialog to configure some parameters of the classifier
        this  = uiconfigure(this)           
    end
    
    methods (Abstract, Static)
        % short Name, appears in GUI as name
        shortName = name                                    
    end
        
    methods 
        function this = show(this)
            % show setup
            fprintf('+++ +++ %s +++ +++\n\n', this.name);
        end
    end
    
    methods (Static)
        function [state msg] = checkSetup
            % check setup, dependencies, e.g. specific toolbox
           state = true;
           msg = 'Ok.';
        end

        function beVisible = shouldBeVisible
            % if the classifier should appear in list of choosable classifiers
            beVisible = true;
        end
    end
    
        
end