classdef janis_knn < janis_classifier & janis_bioinfoWrapper
    % This is just a wrapper to KNN of the Matlab Statistics Toolbox
    
    properties (Constant)
        availableDistances = {  'euclidean'; ...
                                'cityblock'; ...
                                'cosine'; ...
                                'correlation'; ...
                                'Hamming' };        % available distance methods
    end
    
    properties (SetAccess = protected)
        knnParams = {3,'euclidean'};                % default k & distance measure
    end
    
    methods (Static)
        function shortName = name
            shortName = 'K nearest neighbours';
        end
    end
    

    methods
       
        function this = train(this,trainPatterns,trainLabels)
            classifierStruct.trainPatterns = trainPatterns';
            classifierStruct.trainLabels = trainLabels';
            this.classifierStruct = classifierStruct;
        end
        
        function [predictions decFactors] = test(this,testPatterns)
            % calls knnclassify
            predictions = knnclassify(testPatterns', ...
                this.classifierStruct.trainPatterns,this.classifierStruct.trainLabels, ...
                this.knnParams{:});
                % not implemented yet!
                decFactors = struct;
        end
        
        function this = setKNNparams(this, varargin)
            % Set additional Params of the Matlab Knn
            this.knnParams = varargin;
        end
        
        function classifierStruct  = getClassifier(this,trainPatterns,trainLabels)
            classifierStruct.trainPatterns = trainPatterns;
            classifierStruct.trainLabels = trainLabels;
        end
        
        function this = uiconfigure(this)
           kStr=inputdlg('k for k-nearest...','The K',1,{num2str(this.knnParams{1})});
           if ~isempty(str2num(kStr{:})) && rem(str2num(kStr{:}),1) == 0
               this.knnParams{1} = str2num(kStr{:});
               dfIdx = listdlg('PromptString','Select Disctance Function:','SelectionMode','single','ListString',this.availableDistances);
               disFct = this.availableDistances(dfIdx);
               this.knnParams{2} = disFct{:};
           else 
               error('Not a valid value');
           end
        end
                
        function show(this)
            show@janis_classifier(this);
            fprintf('k    =     %d\n', this.knnParams{1});
            fprintf('Distance\n Func.:    %s', this.knnParams{2});
        end
    end
end