% a wrapper to libsvm
%
%   see <http://www.csie.ntu.edu.tw/~cjlin/libsvm/ libsvm> for
%   documentation
classdef janis_libSVM < janis_classifier
    
    properties (SetAccess = protected)
        svmParams = {'-t 0'};           % '-t 0' default value, means: linear function
    end
    
    methods (Static)
        function shortName = name
            shortName = 'libSVM Support Vector Machine';
        end
        
        function    [state msg] = checkSetup
            if exist('svmtrain','file') == 3
                state = true;
                msg = 'libsvm files: ✓';
            else
                state = false;
                msg = 'libsvm not found. Please add to path or download it from http://www.csie.ntu.edu.tw/~cjlin/libsvm/ - compile Matlab Interface and add to search path.';
            end
        end
        
    end
    
    methods
        
        function this = train(this,trainPatterns,trainLabels)
            % calls svmtrain of libsvm
            this.classifierStruct = svmtrain(trainLabels, trainPatterns', this.svmParams{:});
        end
        
        function [predictions decFactors] = test(this,testPatterns)
            % calls svmpredict
            tmp = testPatterns';
            if isempty(this.classifierStruct.ProbA)
                [ignoreOutput, predictions, mse, decVals] = evalc('svmpredict(NaN(size(testPatterns,2),1), tmp, this.classifierStruct)');
            else
                [ignoreOutput, predictions, mse, decVals] = evalc('svmpredict(NaN(size(testPatterns,2),1), tmp, this.classifierStruct, ''-b 1'')');
            end
            decFactors.mse = mse;
            decFactors.decVals = decVals;
        end
        
        function this = setSVMparams(this, varargin)
            % Set additional Params of SVM
            this.svmParams = varargin;
        end
        
        function classifierStruct  = getClassifier(jc,trainPatterns,trainLabels)
            classifierStruct = svmtrain(trainLabels, trainPatterns', jc.svmParams{:});
        end
        
        function this = uiconfigure(this)
            [hlpmsg ignore] = evalc('svmtrain');
            libsvmopts=inputdlg(hlpmsg(93:end),'libsvm-options',1,this.svmParams);
            this.svmParams = libsvmopts;
        end
        
        function show(this)
            show@janis_classifier(this);
            fprintf('libsvm options:\n%s\n', this.svmParams{:});
        end
        
    end
    
    
end