% a wrapper to liblinear
%   see <http://www.csie.ntu.edu.tw/~cjlin/liblinear/ liblinear> for
%   documentation
% 
classdef janis_liblinear < janis_classifier
    
    properties (SetAccess = protected)
        svmParams = {'-s 0'};           % '-s 0' default value, means: linear function
    end
    
    methods (Static)
        
        function shortName = name
            shortName = 'liblinear';
        end
        
        function    [state msg] = checkSetup
            if exist('train','file') == 3
                state = true;
                msg = 'liblinear files: ✓';
            else
                state = false;
                msg = 'Files of liblinear not found. Please download and add to searchpath.';
            end
        end
        
    end
    
    methods
        
        function this = train(this,trainPatterns,trainLabels)
            % calls train of liblinear
            this.classifierStruct = train(trainLabels, sparse(trainPatterns)', this.svmParams{:});
        end
        
        function [predictions decFactors] = test(this,testPatterns)
            % calls predict of liblinear
            tmp = sparse(testPatterns)';
            [ignoreOutput, predictions, mse, decVals] = evalc('predict(NaN(size(testPatterns,2),1), tmp, this.classifierStruct)');
            decFactors.mse = mse;
            decFactors.decVals = decVals;
        end
        
        function this = setSVMparams(this, varargin)
            % set liblinear parameters
            this.svmParams = varargin;
        end
        
        function classifierStruct  = getClassifier(this,trainPatterns,trainLabels)
            classifierStruct = train(trainLabels, trainPatterns', this.svmParams{:});
        end
        
        function this = uiconfigure(this)
            [hlpmsg ignore] = evalc('train');
            libsvmopts=inputdlg(hlpmsg(93:end),'liblinear-options',1,this.svmParams);
            this.svmParams = libsvmopts;
        end
        
        function show(this)
            show@janis_classifier(this);
            fprintf('libsvm options:\n%s\n', this.svmParams{:});
        end
        
    end
    
    
end