% Abstract Class of Patterns which works with SPMfiles
%  e.g. extracting beta and con images from SPM.mats
classdef janis_patternAbstractImg < janis_pattern
    
    properties (SetAccess = protected)
        SPMfile;          % location of SPM.mat
    end
    
    properties (SetAccess = protected)
        imgFiles;         % loactions of imageFiles
    end
    
    methods (Abstract)
        % add new conditions (=SPMfile & condition names)
        addCondition(this,SPMfile,listOfConditions)
    end
    
    methods
        function this = addImage(this,image)
            % add 1 image to existing images
            this.imgFiles{size(this.imgFiles,2)+1} = image;
        end
        
        function this = setSPMfile(this,spmfile)
            % set SPM file
            if exist(spmfile)
                this.SPMfile = spmfile;
            else
                error('SPM file not found!');
            end
        end
        
        function show(jp)
            % show textual represenation of the pattern
            show@janis_pattern(jp);
            fprintf('   SPMfile:         %s\n',jp.SPMfile);
        end
        
        function this = generatePattern(this)
            for imgCount=1:size(this.imgFiles,2)
                images{imgCount} = this.loadImage(this.imgFiles{imgCount});
            end
            % notice image dimension
            this.sourceDimension = cellfun(@(x) size(x),images,'UniformOutput',false);
            fs = getFeatureSelection(this);
            images = fs.reduce(images{:});
            if ~isempty(images)
                this.VoxelValues = images;
                this.isLoaded = true;
            else
                warning('janis:emptyImg','Image was empty, something went wrong');
                this.isLoaded = false;
            end
        end
        
        function jp = freePattern(jp)
            jp.VoxelValues = [];
            jp.isLoaded = false;
        end
        
        function newJp = copy(this)
            newJp = copy@janis_pattern(this);
            newJp.SPMfile = this.SPMfile;
            newJp.imgFiles = this.imgFiles;
        end
        
        function markers = getUniqueMarkers(jp,markers)
            markers.SPMfile  = jp.SPMfile;
            markers.imgFiles = jp.imgFiles;
        end
        
    end
    
    methods (Static)
        
        function image = loadImage(imgFile)
            % load images using spm_read_vols
            try
                image = spm_read_vols(spm_vol(imgFile));
            catch
                error('Loading image failed: %s', imgFile);
            end
        end
        
        
    end
    
end