% according to Wang et al. 2003:
% 
% computes most active rois for each subject as new
% pattern representation
classdef janis_pre_active_rois < janis_preprocessor
    
    properties (SetAccess = protected)
        regions = {};               % List of ROIs
        n = 4;                      % compute min./max. over average of maximum/minimum "n" voxels
    end
    
    methods (Static)
        function desc = description
            desc = 'Most active Peaks of multiple ROIs (according to Wang et al. 2003)';
        end
        function shortName = name
            shortName = 'Peaks of ROIs';
        end
        function    [state msg] = checkSetup
            [state, msg] = janis_pre_roi.checkSetup;
        end
        function beVisible = shouldBeVisible
            beVisible = false;
        end
    end
    
    methods
        
        function bool = isConfigurable(this)
            bool = true;
        end
        
        function this = uiconfigure(this)
            masks = spm_select(inf,'image','Select all (resized) ROIs.');
            avg_n=str2double(inputdlg({'Compute average of n max/min. values?',},'How many max./min. values should be considered for an average min./max. value?',1,{''}));
            this.configure(masks,avg_n);
            
        end
        
        function configure(this,masks,avg_n)
            % for roi in rois : make janis_pre_rois and save.
            this.regions = {};
            for s=1:size(masks,1)
                roiobj = janis_pre_roi;
                roiobj.setServiceMethods(this.serviceMethods)
                this.regions{s} = roiobj.setMask(strtrim(masks(s,:)));
                namecanonical = strtrim(masks(s,:));
                seps = strfind(namecanonical,filesep);
                eidx = strfind(namecanonical,'.');
                this.regions{s}.maskName = namecanonical(seps(end)+1:eidx-1);
                this.n = avg_n;
            end
        end
        
        function patternOut = operate(this, patternIn)
            if ~isempty(this.previousOperations)
                patCount = ...
                    numel(this.previousOperations{1}.reoperate(patternIn))...
                    /numel(this.previousOperations{1}.reoperate(this.regions{1}.mask));
                patCount
            else
                patCount = size(patternIn,1)/numel(this.regions{1}.mask);
            end
            if (patCount > 1)
                patSize = size(patternIn,1)/patCount;
                activn = [];
                for h=1:size(patCount)
                    activn = [activn; this.operate(patternIn(patSize*(h-1)+1:patSize*h,:))];
                end
            else
                activn = zeros(this.n*length(this.regions),size(patternIn,2));
                % 4 every roi:
                for k=1:length(this.regions)
                    this.regions{k}.previousOperations = this.previousOperations;
                    disp(k);
                    for sub=1:size(patternIn,2)
                        temproi = this.regions{k}.operate(patternIn(:,sub));
                        activn((((k-1)*this.n)+1):k*this.n,sub) = this.reduce(temproi);
                    end
                end
            end
            patternOut = activn;
        end
        
        function reduced = reduce(this,roi)
            % compute values
            roi = roi(~isnan(roi));
            roi = sort(roi);
            try
                reduced = roi(end-this.n+1:end);
            catch
                disp('ohoh');
            end
        end
        
        function reoperate(this,pattern)
            error('Not implemented!!!');
        end
        
        function patternOut = reconstruct(preprop, patternIn)
            error('Not implemented!!!');
        end
        
        function plotBarh(this,vals,labels,plotHeader)
            % graphical represenation of feature weights
            h2 = figure;
            barh(vals(:),'group');
            set(gca,'YTick',1:numel(labels));
            set(gca,'YTickLabel',labels);
            title(plotHeader);
        end
        
        function new = copy(this)
            new = copy@janis_preprocessor(this);
            new.regions = this.regions;
            new.n = this.n;
        end
        
        function viewSettings(this)
            viewSettings@janis_preprocessor(this);
            fprintf('  Peaks per Roi:\t%i\n',this.n);
            fprintf('  %i ROIs:\n',length(this.regions));
            for k=1:length(this.regions)
                fprintf('   %i. \t%s\n',k,this.regions{k}.maskName);
            end
        end
        
        
        
    end
    
    
    
end