% Imputation of missing values using kknimpute (Bioinformatics toolbox)
classdef janis_pre_knnimpute < janis_nan_handling & janis_bioinfoWrapper
    
    properties (SetAccess = protected)
        mask;               % mask, that should be applied
    end

    methods (Static)
        function desc = description(this)
            desc = 'Impute unkown values by using knnimpute (Matlab function)'; 
        end
        function shortName = name
            shortName = 'Imputation by KNN';
        end
    end
    
    methods
        
        function patternOut = operate(this, patternIn)
            % Search for NANs and mask them in every pattern
            patternOut = knnimpute(patternIn,5,'Distance','correlation');
        end
        
        function patternOut = reoperate(this, patternIn)
            if any(isnan(patternIn))
                warning('Not implemented! NANs could not be removed.');
            end
            patternOut = patternIn;
        end
        
        function patternOut = reconstruct(this, patternIn)
            % Nothing to do here, because no voxels were removed.
            patternOut = patternIn;
        end
    end
end