% Feature Ranking of Bioninformatics Toolbox
%
classdef janis_pro_featureRanking < janis_processor & janis_bioinfoWrapper
    
    properties (Constant)
        availableCriterions = {   'ttest', ...
            'entropy', ...
            'bhattacharyya', ...
            'roc', ...
            'wilcoxon'};           % available criterions for ranking
    end
    
    properties
        mask;                                           % mask of ranked and choosen features
        rkopts = {'CRITERION','entropy'};               % actual params of the methods
    end
    
    methods (Static)
        function desc = description
            desc = 'Rankfeatures (Matlab function): ranks features according to specified cirterion and uses only most disrciminating voxels.';
        end
        function shortName = name
            shortName = 'Feature Ranking';
        end
    end
    
    methods
        function bool = isConfigurable(this)
            bool = true;
        end
        
        function [train, test] = operate(this,train, test, trainLabels)
            % calls rankfeatures
            [feat,stat] = rankfeatures(train,trainLabels, this.rkopts{:});
            this.mask = zeros(size(train,1),1);
            this.mask(feat) = 1;
            train = train(feat,:);
            test = test(feat,:);
        end
        
        function patternOut = reoperate(this, patternIn)
            patternIn = reshape(patternIn,numel(patternIn),1);
            if size(patternIn,1) < size(this.mask,1) && ~mod(size(this.mask,1),size(patternIn,1))
                for k=1:size(preprop.mask,1)/size(patternIn,1)
                    patternIn = [patternIn; patternIn];
                end
            end
            patternOut = patternIn(this.mask,:);
        end
        
        function patternRecon  = reconstruct(this,patternIn)
            % everthing zero execept the features, that were choosen
            % everything else should get NaN or zero.
            patternRecon = zeros(size(this.mask,1),1);
            patternRecon(logical(this.mask)) = patternIn;
            patternRecon = patternRecon';
        end
        
        function this = uiconfigure(this)
            this.rkopts = {};
            repl=inputdlg({'Number',},'Number of features',1,{''});
            if ~isempty(repl{1})
                this.rkopts{length(this.rkopts)+1} = 'NUMBER';
                this.rkopts{length(this.rkopts)+1} = str2double(repl{1});
            end
            critIdx = listdlg('PromptString','Select Criterion:','SelectionMode','single','ListString',this.availableCriterions);
            if ~isempty(critIdx)
                this.rkopts{length(this.rkopts)+1} = 'CRITERION';
                this.rkopts{length(this.rkopts)+1} = this.availableCriterions{critIdx};
            end
            
        end
        
        function new = copy(this)
            new = copy@janis_processor(this);
            new.rkopts = this.rkopts;
        end
        
        function viewSettings(this)
            viewSettings@janis_processor(this);
            for k=1:2:length(this.rkopts)
                if isnumeric(this.rkopts{k+1})
                    fprintf('Option:\t%s\t- Values: %i\n',this.rkopts{k:k+1});
                else
                    fprintf('Option:\t%s\t- Values: %s\n',this.rkopts{k:k+1});
                end
            end
        end
        
    end
    
    
end