% Features selection iterative on random features
%   using Bionformatics toolbox
%
%
classdef janis_pro_randomFeatures < janis_pro_featureRanking & janis_bioinfoWrapper
    
    properties (Constant)
        availableClassifiers = {   'da', ...
            'knn'};          % available classifiers
    end
    
    properties
        rfopts = {};                                % options
        classOpts = {};                             % classifier options
        nFeats = 100;                               % how many features
    end
    
    
    methods (Static)
        function desc = description
            desc = 'RandomFeatures: Performs embedded classification on random features to determine a good subset of features';
        end
        function shortName = name
            shortName = 'Random Features';
        end
    end
    
    methods
        
        function bool = isConfigurable(this)
            bool = true;
        end
        
        function [train, test] = operate(this,train, test, trainLabels)
            % calls randfeatures
            if ~isempty(this.classOpts)
                feat = randfeatures(train,trainLabels,this.rfopts{:},'ClassOptions',this.classOpts);
            else
                feat = randfeatures(train,trainLabels,this.rfopts{:});
            end
            this.mask = zeros(size(train,1),1);
            this.mask(feat(1:this.nFeats)) = 1;
            train = train(feat(1:this.nFeats),:);
            test = test(feat(1:this.nFeats),:);
        end

        function patternOut = reoperate(this, patternIn)
            patternIn = reshape(patternIn,numel(patternIn),1);
            if size(patternIn,1) < size(this.mask,1) && ~mod(size(this.mask,1),size(patternIn,1))
                for k=1:size(preprop.mask,1)/size(patternIn,1)
                    patternIn = [patternIn; patternIn];
                end
            end
            patternOut = patternIn(this.mask,:);
        end
        
        function patternRecon  = reconstruct(this,patternIn)
            % everthing zero execept the features, that were choosen
            % everything else should get NaN or zero.
            patternRecon = zeros(numel(this.mask),1);
            patternRecon(logical(this.mask)) = patternIn;
            patternRecon = patternRecon';
        end
        
        function this = uiconfigure(this)
            this.rfopts = {};
            repl=inputdlg({'Number',},'Number of features',1,{''});
            if ~isempty(repl{1})
                this.nFeats = str2double(repl{1});
            end
            repl=inputdlg({'Number',},'SubsetSize',1,{'20'});
            if ~isempty(repl{1})
                this.rfopts{length(this.rfopts)+1} = 'SubsetSize';
                this.rfopts{length(this.rfopts)+1} = str2double(repl{1});
            end
            repl=inputdlg({'Number',},'PoolSize',1,{'1000'});
            if ~isempty(repl{1})
                this.rfopts{length(this.rfopts)+1} = 'PoolSize';
                this.rfopts{length(this.rfopts)+1} = str2double(repl{1});
            end
            critIdx = listdlg('PromptString','Select Classifier:','SelectionMode','single','ListString',this.availableClassifiers);
            if ~isempty(critIdx)
                this.rfopts{length(this.rfopts)+1} = 'Classifier';
                this.rfopts{length(this.rfopts)+1} = this.availableClassifiers{critIdx};
            end
            if strcmp(this.rfopts(end),'da')
                tmp = janis_lda;
                tmp = tmp.uiconfigure;
                this.classOpts = tmp.ldaParams;
            elseif strcmp(this.rfopts{end},'knn')
                tmp = janis_knn;
                tmp = tmp.uiconfigure;
                this.classOpts = tmp.knnParams;
            end
        end
        
        function new = copy(this)
            new = copy@janis_processor(this);
            new.rfopts = this.rfopts;
            new.nFeats = this.nFeats;
            new.classOpts = this.classOpts;
        end
        
        function viewSettings(this)
            viewSettings@janis_processor(this);
            fprintf('Option:\t%i\n',this.nFeats);
            for k=1:2:length(this.rfopts)
                if isnumeric(this.rfopts{k+1})
                    fprintf('Option:\t%s\t- Values: %i\n',this.rfopts{k:k+1});
                else
                    fprintf('Option:\t%s\t- Values: %s\n',this.rfopts{k:k+1});
                end
            end
            fprintf('Classifier Options:\n');
            for k=1:length(this.classOpts)
                if isnumeric(this.classOpts{k})
                    fprintf('Values: %i\n',this.classOpts{k});
                else
                    fprintf('Values: %s\n',this.classOpts{k});
                end
            end
        end
        
    end
    
    
end