% Greedy/Fast variant of SVMRFE
%    recurisvely eliminates specific amount of features in every iteration
%
%    using libsvm or (if missing) matlab_SVM 
%
classdef janis_pro_svmrfe < janis_pro_svmweights
    
    methods (Static)
        function desc = description
            desc = 'SVMRFE (greedy / fast): recursive feature elimination - eliminates features during iterated embedded crossivalidation process until specified number of features is reached.';
        end
        function shortName = name
            shortName = 'SVM RFE (greedy / fast)';
        end
    end
    
    methods
        function bool = isConfigurable(this)
            bool = true;
        end
        
        function [train, test] = operate(this,train, test, trainLabels)
            if isempty(strfind(path,'libsvm'))
                jc = janis_matlabSVM;
            else
                jc = janis_libSVM;
            end
            markToRemove = zeros(size(train,1),1);
            count=0;
            % for each computation
            while (this.nFeatures+1<size(train,1)-sum(markToRemove) && round((size(train,1)-sum(markToRemove))*0.05)>0)
                jc = jc.train(train(~markToRemove,:),trainLabels);
                % calcualte (weighted SupportVectors)^2
                if isa(jc,'janis_matlabSVM')
                    svecs = jc.classifierStruct.SupportVectors;
                    coeffs = jc.classifierStruct.Alpha;
                else
                    jc = jc.train(train(~markToRemove,:),trainLabels);
                    svecs = jc.classifierStruct.SVs;
                    coeffs = jc.classifierStruct.sv_coef;
                end
                b = coeffs'*svecs;
                c = sum(b,1);
                wQuad = c.^2;
                % mark the vectors already known to remove
                % then find vectors which small influence
                [wSorted wIdx] = sort(wQuad);
                % calcualte how many features to Remove in this step
                featuresToRemove = round((size(train,1)-sum(markToRemove))*0.05);
                % add the features to remove-list
                tmp = zeros(1,size(markToRemove,1));
                tmp(~markToRemove) = wQuad;
                markToRemove(tmp<=wSorted(featuresToRemove)) = 1;
                if (mod(count,5)==0)
                    disp([num2str(sum(markToRemove)) ' Marked To Remove']);
                    disp([num2str(featuresToRemove) ' features removed']);
                end
                count=count+1;
            end
            
            disp('=========Done==========')
            disp(['Marked To Remove: ' num2str(sum(this.mask))]);
            this.mask = ~markToRemove;
            train = train(this.mask,:);
            test = test(this.mask,:);
        end
        
        function new = copy(this)
            new = copy@janis_processor(this);
            new.mask = this.mask;
            new.nFeatures = this.nFeatures;
        end
        
    end
    
    
end