% Pick features according to feature weights (just one computation, not
% recursive!)
%  tries to use libsvm, otherwise matlab_svm
%
classdef janis_pro_svmweights < janis_processor
    
    properties
        nFeatures = 100;                % features to pick
        mask;                           % mask of picked features
    end
    
    methods (Static)
        function desc = description
            desc = 'SVM-Weights: Weights features according to features weights from embedded crossvalidation (using SVM)';
        end
        function shortName = name
            shortName = 'SVM weights';
        end
        
        function [state msg] = checkSetup
            [bioState bioMsg] = janis_bioinfoWrapper.checkSetup;
            [libsvmState libsvmMsg] = janis_libSVM.checkSetup;
            if libsvmState
                state = libsvmState;
                msg = libsvmMsg;
            elseif bioState
                state = bioState;
                msg = bioMsg;
            else
                state = false;
                msg = 'No SVM algorithm found.';
            end
        end
    end
    
    methods
        function bool = isConfigurable(this)
            bool = true;
        end
        
        function [train, test] = operate(this,train, test, trainLabels)
            this.mask = zeros(size(train,1),1);
            if isempty(strfind(path,'libsvm'))
                jc = janis_matlabSVM;
                jc = jc.train(train,trainLabels);
                svecs = jc.classifierStruct.SupportVectors;
                coeffs = jc.classifierStruct.Alpha;
            else
                jc = janis_libSVM;
                jc = jc.train(train,trainLabels);
                svecs = jc.classifierStruct.SVs;
                coeffs = jc.classifierStruct.sv_coef;
            end
            b = coeffs'*svecs;
            c = sum(b,1);
            wQuad = c.^2;
            [wSorted wIdx] = sort(wQuad);
            this.mask(wIdx(end-this.nFeatures+1:end)) = 1;
            train = train(logical(this.mask),:);
            test = test(logical(this.mask),:);
        end
        
        function patternOut = reoperate(this, patternIn)
            patternIn = reshape(patternIn,numel(patternIn),1);
            if size(patternIn,1) < size(this.mask,1) && ~mod(size(this.mask,1),size(patternIn,1))
                for k=1:size(preprop.mask,1)/size(patternIn,1)
                    patternIn = [patternIn; patternIn];
                end
            end
            patternOut = patternIn(this.mask,:);
        end

        function patternRecon  = reconstruct(this,patternIn)
            patternRecon = zeros(size(this.mask,1),1);
            patternRecon(logical(this.mask)) = patternIn;
            patternRecon = patternRecon';
        end
        
        function this = uiconfigure(this)
            repl=inputdlg({'Number',},'Number of features',1,{''});
            if ~isempty(repl{1})
                this.nFeatures = str2double(repl{1});
            end
        end
        
        function new = copy(this)
            new = copy@janis_processor(this);
            new.mask = this.mask;
            new.nFeatures = this.nFeatures;
        end
        
        function viewSettings(this)
            viewSettings@janis_processor(this);
            fprintf('  Number of voxels:\t%i\n',this.nFeatures);
       end
        
    end
    
    
end