% Default parser for ordinary patterns
classdef janis_simpleParser < janis_patternParser
    
    methods
        
        function pp = janis_simpleParser(classCollection)
            % Constructor
            pp = pp@janis_patternParser(classCollection);
        end
        
        function this = init(this)
            % gets all Patterns from all subjects and generates a Matrix
            % which is used to return the a matrix by specified subjects
            
            % init helper Matrices
            % Patternsize: Voxels X Subjects
            this.patternMat = zeros(this.classCollection.get(1).voxelsPerSub,this.classCollection.patternCount);
            this.patternLabels = zeros(this.classCollection.patternCount,1);
            actualIndex = 1;
            % go through all classes
            for cl=1:this.classCollection.size
                % for each class
                actualClass = this.classCollection.get(cl);
                this.patternLabels(actualIndex:actualIndex+actualClass.size-1) = cl -1;
                % go through all subjects
                for subject=1:actualClass.size
                    actualSubject = actualClass.getMember(subject);
                    vector = actualSubject.get;
                    try
                        this.patternMat(:,actualIndex-1+subject) = vector;
                    catch
                        error(['Dimension missmatch in subject ' actualSubject.description]);
                    end
                end
                actualIndex = actualIndex + actualClass.size;
            end
            this.patternMat = this.applyProcessors(this.patternMat);
        end
        
        function [trainPattern, trainLabels, testPattern, testLabels] = get(this,indices)
            % return specific pattern
            trainPattern = this.patternMat(:,indices);
            testPattern  = this.patternMat(:,~indices);
            trainLabels = this.patternLabels(indices);
            testLabels  = this.patternLabels(~indices);
            for ivp=1:size(this.intraValidationProcessors,2)
                this.intraValidationProcessors{ivp}.preprocessorHistory = this.preprocessors;
                [trainPattern, testPattern] = this.intraValidationProcessors{ivp}.operate(trainPattern,testPattern,trainLabels);
            end
        end
        
        function [trainPattern, trainLabels, testPattern, testLabels] = getReassigned(this,indices,reassignedLabels)
            [trainPattern, trainLabels, testPattern, testLabels] = get(this,indices);
            newTestLabels = nan(length(testLabels),1);
            for k=1:length(reassignedLabels)
                newTestLabels(testLabels==k-1) = reassignedLabels(k);
            end
            testLabels = newTestLabels;
        end
        
        function pattern = reconstructFromProcessors(this,pattern)
            % call reconstruct Functions from Feature Selection methods in
            % reverse order they were applied
            
            % go through reverse
            for ivp=size(this.intraValidationProcessors,2):-1:1
                pattern = this.intraValidationProcessors{ivp}.reconstruct(pattern);
            end
            for ivp=size(this.preprocessors,2):-1:1
                pattern = this.preprocessors{ivp}.reconstruct(pattern);
            end
            origDim = size(this.classCollection.classes{1}.members{1}.VoxelValues);
            pattern = reshape(pattern,origDim);
        end
        
        function labelsAll = getLabels(this)
            % return all the labels
            labelsAll = this.patternLabels;
        end
        
        function labelsAll = getReassignedLabels(this,reassignedLabels)
            labelsAll = this.patternLabels;
            newTestLabels = nan(length(labelsAll),1);
            for k=1:length(reassignedLabels)
                newTestLabels(labelsAll==k-1) = reassignedLabels(k);
            end
            labelsAll = newTestLabels;
        end
        
        function new = copy(this)
            new =  copy@janis_patternParser(this);
        end
    end
end